#!/usr/bin/python 
#-*- coding: UTF-8 -*-
# File: blocks.py 
# Solely for identifying Unicode blocks for unicode characters.
# Based on code from:
# http://stackoverflow.com/questions/243831/unicode-block-of-a-character-in-python
# But updated for 2013.
# Copyright (C) 2013-2016 Peter Murphy <peterkmurphy@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re;

PRIV_USE_BLOCK = 144; 

# If fonts characters are not in an assigned block, then they are assigned to the
# Private Use Area by default.

def block(ch):
    '''
    Return the Unicode block name for ch, or None if ch has no block.

    >>> block(u'a')
    'Basic Latin'
    >>> block(unichr(0x0b80))
    'Tamil'
    >>> block(unichr(0xe0080))
    '''

    assert isinstance(ch, unicode) and len(ch) == 1, repr(ch);
    cp = ord(ch);
    for start, end, name in _blocks:
        if start <= cp <= end:
            return name;
            
def blockbyint(intval):
    for start, end, name in _blocks:
        if start <= intval <= end:
            return name;    

def namefromindex(ith):
    ''' Returns the name of the ith block. '''
    return _blocks[ith][2];

def indexfromname(name):
    ''' Returns the index of a block name. '''
    if name:
        return _blockmap[name];
    else:
        return PRIV_USE_BLOCK;

def numblocks():
    ''' Gets the number of blocks. '''
    return _blocksize;

def _initBlocks(text):
    global _blocks, _blockmap, _blocksize;
    _blocks = [];
    _blockmap = {};
    iter = 0;
    pattern = re.compile(r'([0-9A-F]+)\.\.([0-9A-F]+);\ (\S.*\S)')
    for line in text.splitlines():
        m = pattern.match(line)
        if m:
            start, end, name = m.groups()
            _blocks.append((int(start, 16), int(end, 16), name))
            _blockmap[name] = iter;
            iter += 1;
    _blocksize = len(_blocks);

# retrieved from http://unicode.org/Public/UNIDATA/Blocks.txt
_initBlocks('''
# Blocks-8.0.0.txt
# Date: 2014-11-10, 23:04:00 GMT [KW]
#
# Unicode Character Database
# Copyright (c) 1991-2014 Unicode, Inc.
# For terms of use, see http://www.unicode.org/terms_of_use.html
# For documentation, see http://www.unicode.org/reports/tr44/
#
# Format:
# Start Code..End Code; Block Name

# ================================================

# Note:   When comparing block names, casing, whitespace, hyphens,
#         and underbars are ignored.
#         For example, "Latin Extended-A" and "latin extended a" are equivalent.
#         For more information on the comparison of property values, 
#            see UAX #44: http://www.unicode.org/reports/tr44/
#
#  All block ranges start with a value where (cp MOD 16) = 0,
#  and end with a value where (cp MOD 16) = 15. In other words,
#  the last hexadecimal digit of the start of range is ...0 
#  and the last hexadecimal digit of the end of range is ...F.
#  This constraint on block ranges guarantees that allocations
#  are done in terms of whole columns, and that code chart display
#  never involves splitting columns in the charts.
#
#  All code points not explicitly listed for Block
#  have the value No_Block.

# Property:	Block
#
# @missing: 0000..10FFFF; No_Block

0000..007F; Basic Latin
0080..00FF; Latin-1 Supplement
0100..017F; Latin Extended-A
0180..024F; Latin Extended-B
0250..02AF; IPA Extensions
02B0..02FF; Spacing Modifier Letters
0300..036F; Combining Diacritical Marks
0370..03FF; Greek and Coptic
0400..04FF; Cyrillic
0500..052F; Cyrillic Supplement
0530..058F; Armenian
0590..05FF; Hebrew
0600..06FF; Arabic
0700..074F; Syriac
0750..077F; Arabic Supplement
0780..07BF; Thaana
07C0..07FF; NKo
0800..083F; Samaritan
0840..085F; Mandaic
08A0..08FF; Arabic Extended-A
0900..097F; Devanagari
0980..09FF; Bengali
0A00..0A7F; Gurmukhi
0A80..0AFF; Gujarati
0B00..0B7F; Oriya
0B80..0BFF; Tamil
0C00..0C7F; Telugu
0C80..0CFF; Kannada
0D00..0D7F; Malayalam
0D80..0DFF; Sinhala
0E00..0E7F; Thai
0E80..0EFF; Lao
0F00..0FFF; Tibetan
1000..109F; Myanmar
10A0..10FF; Georgian
1100..11FF; Hangul Jamo
1200..137F; Ethiopic
1380..139F; Ethiopic Supplement
13A0..13FF; Cherokee
1400..167F; Unified Canadian Aboriginal Syllabics
1680..169F; Ogham
16A0..16FF; Runic
1700..171F; Tagalog
1720..173F; Hanunoo
1740..175F; Buhid
1760..177F; Tagbanwa
1780..17FF; Khmer
1800..18AF; Mongolian
18B0..18FF; Unified Canadian Aboriginal Syllabics Extended
1900..194F; Limbu
1950..197F; Tai Le
1980..19DF; New Tai Lue
19E0..19FF; Khmer Symbols
1A00..1A1F; Buginese
1A20..1AAF; Tai Tham
1AB0..1AFF; Combining Diacritical Marks Extended
1B00..1B7F; Balinese
1B80..1BBF; Sundanese
1BC0..1BFF; Batak
1C00..1C4F; Lepcha
1C50..1C7F; Ol Chiki
1CC0..1CCF; Sundanese Supplement
1CD0..1CFF; Vedic Extensions
1D00..1D7F; Phonetic Extensions
1D80..1DBF; Phonetic Extensions Supplement
1DC0..1DFF; Combining Diacritical Marks Supplement
1E00..1EFF; Latin Extended Additional
1F00..1FFF; Greek Extended
2000..206F; General Punctuation
2070..209F; Superscripts and Subscripts
20A0..20CF; Currency Symbols
20D0..20FF; Combining Diacritical Marks for Symbols
2100..214F; Letterlike Symbols
2150..218F; Number Forms
2190..21FF; Arrows
2200..22FF; Mathematical Operators
2300..23FF; Miscellaneous Technical
2400..243F; Control Pictures
2440..245F; Optical Character Recognition
2460..24FF; Enclosed Alphanumerics
2500..257F; Box Drawing
2580..259F; Block Elements
25A0..25FF; Geometric Shapes
2600..26FF; Miscellaneous Symbols
2700..27BF; Dingbats
27C0..27EF; Miscellaneous Mathematical Symbols-A
27F0..27FF; Supplemental Arrows-A
2800..28FF; Braille Patterns
2900..297F; Supplemental Arrows-B
2980..29FF; Miscellaneous Mathematical Symbols-B
2A00..2AFF; Supplemental Mathematical Operators
2B00..2BFF; Miscellaneous Symbols and Arrows
2C00..2C5F; Glagolitic
2C60..2C7F; Latin Extended-C
2C80..2CFF; Coptic
2D00..2D2F; Georgian Supplement
2D30..2D7F; Tifinagh
2D80..2DDF; Ethiopic Extended
2DE0..2DFF; Cyrillic Extended-A
2E00..2E7F; Supplemental Punctuation
2E80..2EFF; CJK Radicals Supplement
2F00..2FDF; Kangxi Radicals
2FF0..2FFF; Ideographic Description Characters
3000..303F; CJK Symbols and Punctuation
3040..309F; Hiragana
30A0..30FF; Katakana
3100..312F; Bopomofo
3130..318F; Hangul Compatibility Jamo
3190..319F; Kanbun
31A0..31BF; Bopomofo Extended
31C0..31EF; CJK Strokes
31F0..31FF; Katakana Phonetic Extensions
3200..32FF; Enclosed CJK Letters and Months
3300..33FF; CJK Compatibility
3400..4DBF; CJK Unified Ideographs Extension A
4DC0..4DFF; Yijing Hexagram Symbols
4E00..9FFF; CJK Unified Ideographs
A000..A48F; Yi Syllables
A490..A4CF; Yi Radicals
A4D0..A4FF; Lisu
A500..A63F; Vai
A640..A69F; Cyrillic Extended-B
A6A0..A6FF; Bamum
A700..A71F; Modifier Tone Letters
A720..A7FF; Latin Extended-D
A800..A82F; Syloti Nagri
A830..A83F; Common Indic Number Forms
A840..A87F; Phags-pa
A880..A8DF; Saurashtra
A8E0..A8FF; Devanagari Extended
A900..A92F; Kayah Li
A930..A95F; Rejang
A960..A97F; Hangul Jamo Extended-A
A980..A9DF; Javanese
A9E0..A9FF; Myanmar Extended-B
AA00..AA5F; Cham
AA60..AA7F; Myanmar Extended-A
AA80..AADF; Tai Viet
AAE0..AAFF; Meetei Mayek Extensions
AB00..AB2F; Ethiopic Extended-A
AB30..AB6F; Latin Extended-E
AB70..ABBF; Cherokee Supplement
ABC0..ABFF; Meetei Mayek
AC00..D7AF; Hangul Syllables
D7B0..D7FF; Hangul Jamo Extended-B
D800..DB7F; High Surrogates
DB80..DBFF; High Private Use Surrogates
DC00..DFFF; Low Surrogates
E000..F8FF; Private Use Area
F900..FAFF; CJK Compatibility Ideographs
FB00..FB4F; Alphabetic Presentation Forms
FB50..FDFF; Arabic Presentation Forms-A
FE00..FE0F; Variation Selectors
FE10..FE1F; Vertical Forms
FE20..FE2F; Combining Half Marks
FE30..FE4F; CJK Compatibility Forms
FE50..FE6F; Small Form Variants
FE70..FEFF; Arabic Presentation Forms-B
FF00..FFEF; Halfwidth and Fullwidth Forms
FFF0..FFFF; Specials
10000..1007F; Linear B Syllabary
10080..100FF; Linear B Ideograms
10100..1013F; Aegean Numbers
10140..1018F; Ancient Greek Numbers
10190..101CF; Ancient Symbols
101D0..101FF; Phaistos Disc
10280..1029F; Lycian
102A0..102DF; Carian
102E0..102FF; Coptic Epact Numbers
10300..1032F; Old Italic
10330..1034F; Gothic
10350..1037F; Old Permic
10380..1039F; Ugaritic
103A0..103DF; Old Persian
10400..1044F; Deseret
10450..1047F; Shavian
10480..104AF; Osmanya
10500..1052F; Elbasan
10530..1056F; Caucasian Albanian
10600..1077F; Linear A
10800..1083F; Cypriot Syllabary
10840..1085F; Imperial Aramaic
10860..1087F; Palmyrene
10880..108AF; Nabataean
108E0..108FF; Hatran
10900..1091F; Phoenician
10920..1093F; Lydian
10980..1099F; Meroitic Hieroglyphs
109A0..109FF; Meroitic Cursive
10A00..10A5F; Kharoshthi
10A60..10A7F; Old South Arabian
10A80..10A9F; Old North Arabian
10AC0..10AFF; Manichaean
10B00..10B3F; Avestan
10B40..10B5F; Inscriptional Parthian
10B60..10B7F; Inscriptional Pahlavi
10B80..10BAF; Psalter Pahlavi
10C00..10C4F; Old Turkic
10C80..10CFF; Old Hungarian
10E60..10E7F; Rumi Numeral Symbols
11000..1107F; Brahmi
11080..110CF; Kaithi
110D0..110FF; Sora Sompeng
11100..1114F; Chakma
11150..1117F; Mahajani
11180..111DF; Sharada
111E0..111FF; Sinhala Archaic Numbers
11200..1124F; Khojki
11280..112AF; Multani
112B0..112FF; Khudawadi
11300..1137F; Grantha
11480..114DF; Tirhuta
11580..115FF; Siddham
11600..1165F; Modi
11680..116CF; Takri
11700..1173F; Ahom
118A0..118FF; Warang Citi
11AC0..11AFF; Pau Cin Hau
12000..123FF; Cuneiform
12400..1247F; Cuneiform Numbers and Punctuation
12480..1254F; Early Dynastic Cuneiform
13000..1342F; Egyptian Hieroglyphs
14400..1467F; Anatolian Hieroglyphs
16800..16A3F; Bamum Supplement
16A40..16A6F; Mro
16AD0..16AFF; Bassa Vah
16B00..16B8F; Pahawh Hmong
16F00..16F9F; Miao
1B000..1B0FF; Kana Supplement
1BC00..1BC9F; Duployan
1BCA0..1BCAF; Shorthand Format Controls
1D000..1D0FF; Byzantine Musical Symbols
1D100..1D1FF; Musical Symbols
1D200..1D24F; Ancient Greek Musical Notation
1D300..1D35F; Tai Xuan Jing Symbols
1D360..1D37F; Counting Rod Numerals
1D400..1D7FF; Mathematical Alphanumeric Symbols
1D800..1DAAF; Sutton SignWriting
1E800..1E8DF; Mende Kikakui
1EE00..1EEFF; Arabic Mathematical Alphabetic Symbols
1F000..1F02F; Mahjong Tiles
1F030..1F09F; Domino Tiles
1F0A0..1F0FF; Playing Cards
1F100..1F1FF; Enclosed Alphanumeric Supplement
1F200..1F2FF; Enclosed Ideographic Supplement
1F300..1F5FF; Miscellaneous Symbols and Pictographs
1F600..1F64F; Emoticons
1F650..1F67F; Ornamental Dingbats
1F680..1F6FF; Transport and Map Symbols
1F700..1F77F; Alchemical Symbols
1F780..1F7FF; Geometric Shapes Extended
1F800..1F8FF; Supplemental Arrows-C
1F900..1F9FF; Supplemental Symbols and Pictographs
20000..2A6DF; CJK Unified Ideographs Extension B
2A700..2B73F; CJK Unified Ideographs Extension C
2B740..2B81F; CJK Unified Ideographs Extension D
2B820..2CEAF; CJK Unified Ideographs Extension E
2F800..2FA1F; CJK Compatibility Ideographs Supplement
E0000..E007F; Tags
E0100..E01EF; Variation Selectors Supplement
F0000..FFFFF; Supplementary Private Use Area-A
100000..10FFFF; Supplementary Private Use Area-B

# EOF''')

if __name__ == '__main__':
    print indexfromname("Private Use Area");
    print block(u'a')
    print block(unichr(0xE000))
    print block(unichr(0xF8FF))
    print block(unichr(0x10000))
    print block(unichr(0x10ffff))
    







#!/usr/bin/python 
#-*- coding: UTF-8 -*-
# File: views.py 
# Copyright (C) 2013-2016 Peter Murphy <peterkmurphy@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os, fnmatch;
import urlparse;
from django.template import Context, RequestContext, loader
from django.http import HttpResponse, Http404, HttpResponseRedirect;
from django.template.response import TemplateResponse;
from django.shortcuts import render_to_response;
from django.conf import settings
from glyphviewer import glyphCatcher, glyphArray, GC_ERRORMSG;
import random;
from settings import STATIC_URL

FONTS_DIR_ADD = "glyphviewer/fonts/";
FIND_LOCAL_NAME = 0;
FIND_LOCAL_RANDOM = 1;
FIND_REMOTE = 2;
localfontfiles = [];
fontnametodirectory = {};
localfontempty = True;
# Loads the documentation.

def doc(request):
    return TemplateResponse(request, 'glyphviewer/doc.html', 
        context=RequestContext(request, {}));

# Gets a list of the local font files (and their internal directory). This
# should never be exposed to the user.

def getLocalFontFiles():
    ''' Initialises localfontfiles to the list of fonts in the static directories,
        if there are any fonts to be found. This saves us reinitialising the
        same array.
    '''
    global localfontfiles;
    global fontnametodirectory;
    global localfontempty;
    if localfontfiles != []:
        localfontempty = False;
        return (localfontfiles, fontnametodirectory,);
    for i in [settings.STATIC_ROOT]: 
        font_dir = os.path.join(i, FONTS_DIR_ADD);
        listdir = os.listdir(font_dir);
        filtereddir = filter(lambda(x): fnmatch.fnmatch(x, '*.ttf') or
            fnmatch.fnmatch(x, '*.otf') or fnmatch.fnmatch(x, '*.woff'), 
            listdir);
        for j in filtereddir:
            if not fontnametodirectory.has_key(j):
                localfontfiles.append(j);
                fontnametodirectory[j] = font_dir;
    localfontfiles.sort();
    if localfontfiles != []:
        localfontempty = False;
    else:
        localfontempty = True;
    return (localfontfiles, fontnametodirectory,);


def index(request):

# DANGER! We must be careful of the following things:
# 1. Parsing the GET request from the user. It takes the form.
# Location=[Local|Remote]&fonturl=fonturlval&fontname=fontnameval&blocks=blocks
# 2. Allowing the user to access arbitrary files on the server.
# 3. Handling errors.
# We start by handling the GET request.

# 'shtables': whether tables of Unicode characters are present in the result
# not.

    if "shtables" not in request.GET:
        shtables = False;
    else:
        shtables = True;

# "blocks": whether Unicode characters should be split up by blocks.

    if "blocks" not in request.GET:
        blocks = False;
    else:
        blocks = True;
        
# "locchoice": indicates the location where to find the font. Values are
# FIND_LOCAL_NAME: from the fontname field: a local font chosen by the user.
# FIND_LOCAL_RANDOM: from the fontname field: a local font chosen randomly.
# FIND_REMOTE: from the fonturl field: a remote font chosen by URL.

    if "Location" in request.GET:
        if request.GET["Location"] == "Local":
            locchoice = FIND_LOCAL_NAME;
        else:
            locchoice = FIND_REMOTE;
    else:
        if "fonturl" in request.GET:
            locchoice = FIND_REMOTE;
        else:
            locchoice = FIND_LOCAL_RANDOM;

# "fontlocal": the value of the drop down box containing local values.
# "fontremote": the value of the remote url field.

    fontlocal = request.GET.get("fontname", None);
    fontremote = request.GET.get("fonturl", None);
    
# "localfontfiles" is a list of local font files.
# "fontnametodirectory" is a key-value pairs of font files to their directory.
    
    (localfontfiles, fontnametodirectory,) = getLocalFontFiles();
    
# The "localfontdir_url" is an URL to where font files are stored. 

    localbase_url = urlparse.urljoin("http://" + request.META["HTTP_HOST"],
        STATIC_URL);
    localfontdir_url = urlparse.urljoin(localbase_url, FONTS_DIR_ADD);
    print localfontdir_url;
    
# Now we have to set:
# (i) 'is_remote': False if the returned page has "Local" selected; True for 
# "Remote".
# (ii) 'chosenitem': the value to set the "File" drop down list.
# (iii) 'remoteurl': the value to set the "URL" text box.
# (iv) 'fetchpath': the URL which is used to analyse the font (whereever it is).
# (v) 'displayfont': what font info is actually displayed to the user.

    
    if locchoice == FIND_LOCAL_NAME:
        is_remote = False;
        chosenitem = fontlocal;
        remoteurl = "";
        fetchpath = urlparse.urljoin(localfontdir_url, fontlocal); 
        displayfont = fontlocal; 
        bCheckCORS = False        
    elif locchoice == FIND_REMOTE:
        is_remote = True;
        chosenitem = "";
        remoteurl = fontremote;
        fetchpath = fontremote; 
        displayfont = fontremote;
        bCheckCORS = True
    else: # locchoice == FIND_LOCAL_RANDOM:
        is_remote = False;
        bCheckCORS = False
        random.seed();
        if len(localfontfiles):
            chosenitem = random.choice(localfontfiles); 
        else:
            chosenitem = "";
        remoteurl = "";
        fetchpath = urlparse.urljoin(localfontdir_url, chosenitem);
        displayfont = chosenitem;

# Now we analyse the font!
    
    ourtuples = glyphCatcher(fetchpath, blocks, settings.DEBUG, bCheckCORS);
    
# The 'ourerror' variable contains the error message (if relevant).    
    
    ourerror = ourtuples[0];
    
# The 'ourheader' variable contains the header information for the font.    
    
    ourheader = ourtuples[1];
    
# The 'ourglyphs' variable contains the characters for the font (if relevant).    
    
    ourglyphs = ourtuples[2];
    return TemplateResponse(request, 'glyphviewer/index.html', 
        context=RequestContext(request, {'ourheader': ourheader, 
        'ourglyphs': ourglyphs,'reslistdir':localfontfiles,
        'chosenitem': chosenitem, 'displayfont': displayfont, 'blocks': blocks,
        'ourerror': ourerror, 'ermsg': GC_ERRORMSG[ourerror], 
        'fontpath': fetchpath, 'shtables': shtables,
        'remoteurl':remoteurl, 'is_remote':is_remote, 'localfontempty': localfontempty}));
    





#!/usr/bin/python 
#-*- coding: UTF-8 -*-
# File: glyphviewer.py 
# Goal - to read the unicode points for a font and pass it as a map.
# Copyright (C) 2013-2016 Peter Murphy <peterkmurphy@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from blocks import blockbyint, namefromindex, indexfromname, numblocks;
from fontTools import ttLib;
import os.path;
import urllib;
import datetime;
import urllib2

PLAT_STANDARD_ID = 3; # Never changes.
PLAT_UNIENC_ID = 1; # We're only interested in Unicode encodings.
PLAT_UCS4_ID = 10; # We're only interested in Unicode encodings.
ALL_CHAR_BLOCK = u"All Unicode Characters";
FONT_NAME_PLATID = 1; # Used with getting Font name info. Do not change.
FONT_NAME_ENCID = 0; # Used with getting Font name info. Do not change.

# The following constants are taken from
# http://developer.apple.com/fonts/TTRefMan/RM06/Chap6name.html

FONT_NAME_CPY = 0; # Copyright notice.
FONT_NAME_FFY = 1; # Font Family. 
FONT_NAME_SFY = 2; # Font Subfamily.
FONT_NAME_USI = 3; # Unique subfamily identification.
FONT_NAME_FNM = 4; # Full name of the font.
FONT_NAME_VSN = 5; # Version of the name table.
FONT_NAME_PSC = 6; # PostScript name of the font. 
FONT_NAME_TMK = 7; # Trademark notice.
FONT_NAME_MFC = 8; # Manufacturer name.
FONT_NAME_DES = 9; # Designer; name of the designer of the typeface.
FONT_NAME_DCS = 10; # Description; description of the typeface. 
FONT_NAME_URL = 11; # URL of the font vendor (e.g., http://, ftp://). 
FONT_NAME_URD = 12; # URL of the font designer (e.g., http://, ftp://)
FONT_NAME_LDS = 13; # Description of how the font may be legally used.
FONT_NAME_LIU = 14; # License information URL

# These are error codes from GlyphCatcher. I like to give numeric values - 
# blame my C/C++ heritage.

GC_NOERROR = 0;
GC_NORESOURCE = 1; # Can't even open the resource.
GC_RETFAIL = 2; # Can't retrive the resource.
GC_TIMEOUT = 3; # Timeout
GC_TOOBIG = 4; # Resource too big.
GC_NOTAFONT = 5; # Not a font.
GC_NOHEADER = 6; # No font header.
GC_NOUNICODE = 7; # Symbol font or other dodginess.
GC_OTHERERROR = 8; # Exception thrown somewhere else or other.
GC_WARNCORS = 9; # Problem with CORS and remote resources.

# These are the limitations on the function, so it won't accept some
# excessively sized object.

FONT_MAX_SIZE = 3*1024*1024; # 3 Meg - probably more than necessary.
FONT_TIMEOUT = 120; # 120 seconds - probably more than necessary.
FONT_TIMEOUT_DELTA = datetime.timedelta(seconds=FONT_TIMEOUT);

# It's quicker to interpolate error strings outside the template.

GC_ERRORMSG = ["", # No error message for status 0,
    "It cannot be opened.",
    "It cannot be accessed or retrieved.",
    "It too more than %d seconds to retrieve it." % FONT_TIMEOUT,
    "It exceeds %d bytes in size. " % FONT_MAX_SIZE,
    "It does not appear to be a legitimate font file.",
    "There is no font header found.",
    "The font does not appear to be a legitimate Unicode font. Did you select a 'Symbol' or \
    'Wingdings' font for processing? If so, shame on you.",
    "We don't know what's happened, but some error has arisen. Sorry about that.",
    "It is possible to analyse the font header, but CORS prevents the \
    display of the glyphs in the font. In this case, the browser will display characters \
    using some fall-back font, which should not be used as a guide to the appearance of the \
    font selected here."];


# For characters that shouldn't be printed.

DODGY = "Invalid HTML Characters"

def isvalhtml(ichar):
    ''' Input: numerical value for character. Returns True if it corresponds
    to a legitimate HTML character; False otherwise.
    
    Note 1: not tested for characters beyond 65536.
    Note 2: makes 0 a dodgy character as well.
    '''
# It's easier to mark line break characters as illegitimate.
#
#    if ((ichar < 9) or (ichar == 11)):
#        return False;
#    if ((ichar >= 14) and (ichar <= 31)):
#        return False;

    if (ichar <= 31):
        return False;
    if ((ichar >= 127) and (ichar <= 159)):
        return False;
    if ((ichar == 65535) or (ichar == 65534)):
        return False;
    if ((ichar >= 64976) and (ichar <= 65007)):
        return False;
    return True;


# Here's the code that polices the size limitation.

class TooBigException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

# Here's the code that polices the time limitation.

class TimeoutException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

def sizecheck(count, blockSize, totalSize):
    sizeread = 0;
    blocksize = blockSize;
#    if datetime.datetime.now() - nowtime > FONT_TIMEOUT_DELTA:
#        raise TimeoutException;    
    if totalSize > FONT_MAX_SIZE:
        raise TooBigException(totalSize);
    if count * blocksize > FONT_MAX_SIZE:
        raise TooBigException(count * blocksize);


class glyphArray():
    ''' Represents a array of unicode character with the name of its block
        (or "All Unicode Characters" if there is no particular block. 
    '''
    def __init__(self, blockName, codePoints):
        self.blockName = blockName;
        self.codePoints = codePoints;
        
    def __unicode__(self):
        return self.blockName + ": " + unicode(self.codePoints) + u"\n";
        
    def __str__(self):
        return unicode(self);

class fontHeader():
    ''' Represents the header of a font - in a form presentable in HTML. '''
    def __init__(self, ournametable):
        def stringex(font_name_id):
            our_data = ournametable.getName(font_name_id, FONT_NAME_PLATID,
                FONT_NAME_ENCID);
            self.ournametable = ournametable
            if our_data:
                return our_data.string;
            else:
                return None;
                
        self.copyright = stringex(FONT_NAME_CPY); # Copyright notice.
        self.fontfamily = stringex(FONT_NAME_FFY); # Font Family. 
        self.subfamily = stringex(FONT_NAME_SFY); # Font Subfamily.
        self.usi = stringex(FONT_NAME_USI); # Unique subfamily identification.
        self.fullname = stringex(FONT_NAME_FNM); # Full name of the font.
        self.version = stringex(FONT_NAME_VSN); # Version of the name table.
        self.ps = stringex(FONT_NAME_PSC); # PostScript name of the font. 
        self.trademark = stringex(FONT_NAME_TMK); # Trademark notice.
        self.manufac = stringex(FONT_NAME_MFC); # Manufacturer name.
        self.designer = stringex(FONT_NAME_DES); # Designer name.
        self.description = stringex(FONT_NAME_DCS); # Description.
        self.urlfontven = stringex(FONT_NAME_URL); # font vendor URL. 
        self.urlfontdes = stringex(FONT_NAME_URD); # font designer URL.
        self.descrip = stringex(FONT_NAME_LDS); # Legal use information.
        self.licenseurl = stringex(FONT_NAME_LIU); # License information URL.
        
    def __unicode__(self):
        return self.fullname;
        
    def __str__(self):
        return unicode(self);

def glyphCatcher(fontName, bStoreInBlocks = False, debugMode = False, bCheckCORS=False):
    ''' This is the main meat of the glyphviewer application. The function
        takes a font, extracts the header and a list of the glyphs it
        supports. Or attempts to...
        
        Return values are of the form:
        (error code, fontHeader, sequence of glyphArray,)
        
        Arguments:
        fontName: the file name or URL for a font.
        bStoreInBlocks: if False, the function returns a stored sequence of
        integers for the glyphs. if True, the function returns a sequence of
        sequences, each corresponding to blocks in Unicode. 
        debugMode: if True, fontName can be a file name. If False, fontName
        must be a URL. This is generally the same value as settings.DEBUG.
        bCheckCORS: whether to check for Cross Origin Resource Sharing
    '''

    def _cleanup(arg):
        ''' Only called in the routine. '''
        if bUseTempFile:
            if os.path.exists(resourceName):
                os.unlink(resourceName);
        return arg;


    DodgyGlyphArray = glyphArray(DODGY, []);

    bUseTempFile = False;
    retrievalInfo = None;
    bCORSBlues = False
    
# Is the font a local resource, or accessed by http?        
    
    if os.path.exists(fontName) & debugMode:
        resourceName = fontName;
    else:
        opener = urllib.FancyURLopener({})
        try:
            print ("Wow")
            print (fontName)
            nowtime = datetime.datetime.now();
            retrievalInfo = opener.retrieve(fontName, reporthook=sizecheck)
            if bCheckCORS:
                if "Access-Control-Allow-Origin" not in retrievalInfo[1]:
                    bCORSBlues = True
                elif retrievalInfo[1]["Access-Control-Allow-Origin"] != "*":
                    bCORSBlues = True
            
        except TooBigException:
            return (GC_TOOBIG, None, None,);
        except TimeoutException:
            return (GC_TIMEOUT, None, None,);
        except:
            return (GC_RETFAIL, None, None);
        resourceName = retrievalInfo[0];
        bUseTempFile = True;
    
    try:
        ourFont = ttLib.TTFont(resourceName);
    except (ttLib.TTLibError):
        print ttLib.TTLibError.message
        return _cleanup((GC_NOTAFONT, None, None,));
    except:
        return _cleanup((GC_OTHERERROR, None, None,));
    
# Since TTFont (and WOFFFont) are "lazy", non-fonts may not be detected until
# an explicit attempt is done to read the cmap table.

    if bCORSBlues:
        ourgoodoutcome = GC_WARNCORS
    else:
        ourgoodoutcome = GC_NOERROR

    try:
        cmaptable = ourFont['cmap'].getcmap(PLAT_STANDARD_ID, PLAT_UCS4_ID);
        if cmaptable is None:
            cmaptable = ourFont['cmap'].getcmap(PLAT_STANDARD_ID, PLAT_UNIENC_ID);
            if cmaptable is None:
                return _cleanup((GC_NOUNICODE, None, None,));
        ourheader = fontHeader(ourFont['name']);
    except:
        return _cleanup((GC_NOHEADER, None, None,));
    if bStoreInBlocks:
        blockcontents = [[] for i in range(numblocks())];
        for i in cmaptable.cmap.keys():
            if not isvalhtml(i):
                DodgyGlyphArray.codePoints.append(i);
            else:
                (blockcontents[indexfromname(blockbyint(i))]).append(i);
        [i.sort() for i in blockcontents];
        blockarrangements = [DodgyGlyphArray];
        for i in range(numblocks()):
            if len(blockcontents[i]) > 0:
                blockarrangements.append(glyphArray(namefromindex(i), 
                    blockcontents[i]));
        return _cleanup((ourgoodoutcome,ourheader,blockarrangements,));
    else:
        blockcontents = list(cmaptable.cmap.keys());
        blockcontents.sort();
        ExcellentGlyphArray = glyphArray(ALL_CHAR_BLOCK,[])
        for i in blockcontents:
            if isvalhtml(i):
                ExcellentGlyphArray.codePoints.append(i);                
            else:
                DodgyGlyphArray.codePoints.append(i);
        return _cleanup((ourgoodoutcome, ourheader,[DodgyGlyphArray, ExcellentGlyphArray],));

if __name__ == '__main__':
    print glyphCatcher("", True);
    content = glyphCatcher("http://themes.googleusercontent.com/static/fonts/robotoslab/v1/y7lebkjgREBJK96VQi37ZobN6UDyHWBl620a-IRfuBk.woff", False); #""http://127.0.0.1:8000/static/glyphviewer/fonts/Essays1743.woff", True);
    header = content[1];
    print header; 
    print header.urlfontven 
    print header.urlfontdes 
    body = content[2];
    for i in body:
        print i;
 #   content = glyphCatcher("http://127.0.0.1:8000/static/glyphviewer/fonts/Essays1743.woff", False);
 #   body = content[2];
 #   for i in body:
 #       print i;        



#!/usr/bin/env python

from setuptools import setup

setup(name='glyphviewer',
    version='0.4.1',
    description='A Django app for examining web fonts and the characters they contain.',
    author='Peter Murphy',
    author_email='peterkmurphy@gmail.com',
    url='http://pypi.python.org/pypi/glyphviewer/',
    packages=['glyphviewer', 'glyphviewer.templatetags'],
    package_data={
        'glyphviewer': [
            'templates/glyphviewer/*.html',
            'static/glyphviewer/fonts/*.woff',
        ],
    },
    keywords = 'font glyph web woff ttf otf TrueType OpenType Django',
    license='LICENSE.txt',
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: Other Environment",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Other Audience",
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',        
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Operating System :: OS Independent",
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',        
        'Topic :: Artistic Software',
        'Topic :: Education',
        'Topic :: System :: Console Fonts',
        'Topic :: Text Processing :: Fonts',
        'Topic :: Text Processing :: Linguistic',
        'Topic :: Utilities',
        ],
    long_description=open('README.rst').read(),
    install_requires = ["Django >= 1.0", "fonttools >= 3.0"],
)



#!/usr/bin/python 
#-*- coding: UTF-8 -*-
# File: glyphtable.py 
# Goal - to provide a nice template tag to represent tables of glyphs.
# Copyright (C) 2013, Peter Murphy <peterkmurphy@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from glyphviewer.glyphviewer import glyphCatcher, glyphArray, fontHeader, DODGY;
from django import template;
from django.utils.safestring import mark_safe

DEFAULT_GLYPHTABLE_SIZE = 16; # 16 is a nice round number.

# The following constants are nice for interpolation.

TABSUM = u"<table style=\"table-layout: fixed; width:100%;\" class=\"glyphtable table-bordered table-styled\">\n";
TABCAP = u"<caption class=\"glyphcaption\"><strong>%s</strong></caption>\n<tbody \
class=\"glyphtbody\">\n";

#TABCAP = u"<thead><tr><th rowspan=\"8\">%s</th></tr></thead>\n<tbody \
#class=\"glyphtbody\">\n";


#<td rowspan="number"> 


CELLMAKER = "<td class = \"glyphtd\"><p class = \"glyphchar text-center\">&#%s;<br />\n\
<p class = \"glyphnum text-center\"><strong>U+%04X</strong></p></td>";
REMAINDER = "<td class = \"glyphempty\"><p class=\"glyphchar text-center\">&nbsp;</p>\
<p class=\"glyphnum text-center\">&nbsp;</p></td>";

register = template.Library();

@register.filter(name="glyphtable")
def glyphtable(value, arg = DEFAULT_GLYPHTABLE_SIZE):
    ''' Takes an argument (value) that is assumed to be of type glyphArray.
    The function returns HTML that represents all the glyphs as a table. The
    arg argument indicates the number of columns for the table.
    
    Note: many of the generated entries are given HTML classes - all the 
    better for CSS styling.    
    
    '''
    try:
        
# Note: for HTML validity, font characters that are control or line break are
# not represented in table form nor displayed, but are listed by their 
# numerical code points.
        
        if (value.blockName == DODGY):
            if (len(value.codePoints) == 0):
                return "";
            output = "<p><em>Note: the following numbers listed are the code ";
            output += "point values of characters in the font that would ";
            output += "normally not be printed, such as control characters. ";
            output += "For that reason, they are represented separately.</em></p>\n";
            output += "<ul>\n";
            for i in value.codePoints:
                output += "<li>U+%04X</li>\n" % i;
            output += "</ul>\n";            
            return mark_safe(output);
        
# Any exception - any at all - returns an empty string.

        blocksize = len(value.codePoints);
        numfullrows = blocksize / arg;
        remainder = (blocksize % arg);
        output = TABSUM + TABCAP % value.blockName;
        
# It's more efficient to create output whole rows.         
        
        for i in range(numfullrows):
            output += "<tr class = \"glyphtr\">"
            for j in range(arg):
                ournumber = value.codePoints[j + i * arg];
                output += CELLMAKER % (str(ournumber), ournumber,);
            output += "</tr>\n";

# And then deliver the last row at the end (if any).

        if remainder == 0:
            output += "</tbody>\n</table>\n";
            return mark_safe(output);
        output += "<tr class = \"glyphtr\">"
        for j in range(remainder):
            ournumber = value.codePoints[j + numfullrows * arg]; 
            output += CELLMAKER % (str(ournumber), ournumber,);
        for j in range(arg - remainder):
            output += REMAINDER;
        output += "</tr>\n";
        output += "</tbody>\n</table>\n";
        return mark_safe(output);

# And if it all turns to shit...

    except:
        return "";
glyphtable.is_safe = False

@register.inclusion_tag('glyphviewer/showheader.html')
def showheader(value):
    return {'value': value}






